local _, addon = ...

-- functions
local print, select, tostring, type = print, select, tostring, type
local format = string.format
-- constants
-- frames
-- libs

addon.EVENT_BAG_SCAN_STARTED = "BAG_SCAN_STARTED"
addon.EVENT_BAG_SCAN_FINISHED = "BAG_SCAN_FINISHED"
addon.EVENT_CHARACTER_ADDED = "CHARACTER_ADDED"
addon.EVENT_CHARACTER_CHANGED = "CHARACTER_CHANGED"
addon.EVENT_CHARACTER_REMOVED = "CHARACTER_REMOVED"
addon.EVENT_KEYSTONE = "KEYSTONE"
addon.EVENT_GUILD_INFO_CHANGED = "GUILD_INFO_CHANGED"
addon.EVENT_PLAYER_INFO_CHANGED = "PLAYER_INFO_CHANGED"
addon.EVENT_OPTION_CHANGED = "OPTION_CHANGED"
addon.EVENT_RECORD = "RECORD"
addon.EVENT_RECORD_SCAN_STARTED = "RECORD_SCAN_STARTED"
addon.EVENT_RECORD_SCAN_FINISHED = "RECORD_SCAN_FINISHED"

addon.EVENT_INITIALISED = "INITIALISED"
addon.EVENT_CORE_INITIALISED = "CORE_INITIALISED"
addon.EVENT_BROKER_INITIALISED = "BROKER_INITIALISED"

addon.isBFA = select(4, GetBuildInfo()) >= 80000

function addon:Print(...)
    local obj
    for i = 1, #... do
        obj = select(i, ...)
        if obj and type(obj) == "table" then
            print(format("|cff%sBK:|r", "6699CC"))
            DevTools_Dump(obj)
        elseif obj then
            print(format("|cff%sBK:|r %s", "6699CC", type(obj) == "string" and format(obj, ...) or tostring(obj)))
        end
    end
end

function addon:Debug(...)
    if not ... or not self.options or self.options.debug ~= true then return end

    return self:Print(...)
end

function addon:Initialise(db)
    self:Debug("Initialising.")

    self.db = db

    self.db.characters = self.db.characters or {}
    -- migrate old "pools" system into new db format
    if self.db.pools ~= nil then
        if self.db.pools.player ~= nil then
            self.db.characters = self.db.pools.player
            self.db.pools.player = nil
        end
        self.db.pools = nil
    end

    local key = self:GetCharacterKey()
    local playerInfo = self:GetPlayerInfo()
    self.db.characters[key] = self.db.characters[key] or playerInfo

    self.player = self.db.characters[key]
    for k, v in pairs(self.player) do
        if self.player[k] ~= playerInfo[k] then
            self:Debug("Player %s has changed (%s -> %s); updating database.", k, self.player[k], playerInfo[k])
            self.player[k] = playerInfo[k]
        end
    end

    self:Fire(self.EVENT_CORE_INITIALISED)
end

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    addon:Debug("Ready.")
end)
