local _, addon = ...

-- functions
local select, tonumber = select, tonumber
local format, split = string.format, string.split
local C_ChallengeMode = C_ChallengeMode
local C_ChallengeMode_GetMapInfo = addon.isBFA and C_ChallengeMode.GetMapUIInfo or C_ChallengeMode.GetMapInfo
local GetLocale = GetLocale
-- constants
local ITEM_QUALITY_COLORS = CUSTOM_ITEM_QUALITY_COLORS or ITEM_QUALITY_COLORS
-- frames
-- libs

addon.shorthandDungeonNames = GetLocale() ~= "enUS" and {} or {
    -- Legion
    [197] = "Eye",
    [198] = "DHT",
    [199] = "BRH",
    [200] = "HoV",
    [206] = "Nelth",
    [207] = "Vault",
    [208] = "Maw",
    [209] = "Arcway",
    [210] = "Court",
    [227] = "Lower",
    [233] = "Cath",
    [234] = "Upper",
    [239] = "Seat",

    -- Battle for Azeroth
    [244] = "Atal", -- Atal'dazar
    [245] = "Freehold", -- Freehold
    [246] = "Tol", -- Tol Dagor
    [247] = "Motherlode", -- THE MOTHERLODE
    [248] = "Manor", -- Waycrest Manor
    [249] = "Kings", -- Kings' Rest
    [250] = "Temple", -- Temple of Sethraliss
    [251] = "Underrot", -- The Underrot
    [252] = "Shrine", -- Shrine of the Storm
    -- [000] = "Siege", -- Siege of Boralus
}

function addon:GetMapName(mapId)
    if self.options.useShorthand and self.shorthandDungeonNames[mapId] then
        return self.shorthandDungeonNames[mapId]
    end

    return select(1, C_ChallengeMode_GetMapInfo(mapId))
end

function addon:GetKeystoneInformation(link)
    local info = {split(":", link)}

    local instanceMapId = tonumber(info[addon.isBFA and 3 or 2])
    local instanceName = self:GetMapName(instanceMapId)

    local level = tonumber(info[addon.isBFA and 4 or 3])
    local affix1 = tonumber(info[addon.isBFA and 5 or 4])
    local affix2 = tonumber(info[addon.isBFA and 6 or 5])
    local affix3 = tonumber(info[addon.isBFA and 7 or 6])

    return link, instanceMapId, instanceName, level
end

local function GetItemQualityColour(qualityId)
    return ITEM_QUALITY_COLORS[qualityId] and ITEM_QUALITY_COLORS[qualityId].hex or "|cffFFFFFF"
end

function addon:GetKeystoneLevelColour(level)
    if not self.options.colourKeystoneLevelsByDifficulty then
        return GetItemQualityColour(2)
    end

    if level > 17 then return GetItemQualityColour(6) end
    if level > 14 then return GetItemQualityColour(5) end
    if level > 9  then return GetItemQualityColour(4) end
    if level > 6  then return GetItemQualityColour(3) end
    if level > 3  then return GetItemQualityColour(2) end
    return GetItemQualityColour(1)
end

function addon:GetKeystoneDisplayText(link, useColour)
    local _, _, instanceName, level = self:GetKeystoneInformation(link)

    if useColour then
        local colour = self:GetKeystoneLevelColour(level)

        return format("|cff%s%s|r %s%d|r", "ffffff", instanceName, colour, level)
    end

    return format("%s %d", instanceName, level)
end

-- TODO: Should be moved to the file where records are handled but for now...
function addon:GetRecordDisplayText(record, useColour)
    local mapId, level = unpack(record or {})
    if not mapId then return end

    local instanceName = self:GetMapName(mapId)
    if useColour then
        local colour = self:GetKeystoneLevelColour(level)

        return format("|cff%s%s|r %s%d|r", "ffffff", instanceName, colour, level)
    end

    return format("%s %d", instanceName, level)
end
