local _, addon = ...

-- functions
local format = string.format
local C_Timer = C_Timer
local C_Timer_NewTimer = C_Timer.NewTimer
local CreateFrame = CreateFrame
local GetContainerItemID = GetContainerItemID
local GetContainerItemLink = GetContainerItemLink
local GetContainerNumSlots = GetContainerNumSlots
-- constants
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
-- frames
-- libs

local KEYSTONE_ITEM_ID = 138019

addon:RegisterCallback(addon.EVENT_BAG_SCAN_STARTED, function()
    addon:Debug("Scanning for keystones in bags...")
end)

addon:RegisterCallback(addon.EVENT_BAG_SCAN_FINISHED, function(_, bagId, slotId, link)
    addon:Debug(link and format("Found `%s`.", link) or "Could not find keystone in bags.")
    addon:SetCharacterKeystone(addon:GetCharacterKey(), link)
end)

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    local scanDelay = 1

    local function scanCallback()
        addon:Fire(addon.EVENT_BAG_SCAN_STARTED)

        for bagId = 0, NUM_BAG_SLOTS do
            for slotId = 0, GetContainerNumSlots(bagId) do
                local itemId = GetContainerItemID(bagId, slotId)
                if KEYSTONE_ITEM_ID == itemId then
                    local link = GetContainerItemLink(bagId, slotId)
                    addon:Fire(addon.EVENT_BAG_SCAN_FINISHED, bagId, slotId, link)
                    return
                end
            end
        end

        addon:Fire(addon.EVENT_BAG_SCAN_FINISHED)
    end

    local scanTimer
    local function onEvent(self, event, ...)
        local itemName = GetItemInfo(KEYSTONE_ITEM_ID)
        if not itemName then
            -- The game client currently doesn't have information about the keystone item.  Abort the scan because any results we get cannot be trusted to be accurate.
            return
        end

        if event == "GET_ITEM_INFO_RECEIVED" then
            local arg1 = select(1, ...)
            if arg1 ~= KEYSTONE_ITEM_ID then return end
        end

        if scanTimer then
            scanTimer:Cancel()
        end

        scanTimer = C_Timer_NewTimer(scanDelay, scanCallback)
    end

    local f = CreateFrame("frame")
    f:SetScript("OnEvent", onEvent)
    f:RegisterEvent("BAG_UPDATE_DELAYED")
    f:RegisterEvent("CHALLENGE_MODE_COMPLETED")
    f:RegisterEvent("CHALLENGE_MODE_RESET")
    f:RegisterEvent("CHALLENGE_MODE_START")
    f:RegisterEvent("GET_ITEM_INFO_RECEIVED")

    function addon:BagScannerOnEvent()
        onEvent(f, "BAG_UPDATE_DELAYED")
    end

    local itemName = GetItemInfo(KEYSTONE_ITEM_ID)
    if itemName ~= nil then -- The game client has information about the item cached
        onEvent(f, "GET_ITEM_INFO_RECEIVED", KEYSTONE_ITEM_ID)
    end
end)
